/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

/*
 * TODO: gdamsampleeditor should have a "samples_updated" signal to trap
 *       and queue a redraw. 
 */


#include "gdamsampleeditorskin.h"

static void
gdam_sample_editor_skin_init(GdamSampleEditorSkin* editor)
{
}

static void
gdam_sample_editor_skin_class_init(GdamSampleEditorSkinClass* editor_class)
{
}

GtkType gdam_sample_editor_skin_get_type()
{
	static GtkType sample_editor_skin_type = 0;
	if (!sample_editor_skin_type) {
		static const GtkTypeInfo sample_editor_skin_info = {
			"GdamSampleEditorSkin",
			sizeof(GdamSampleEditorSkin),
			sizeof(GdamSampleEditorSkinClass),
			(GtkClassInitFunc) gdam_sample_editor_skin_class_init,
			(GtkObjectInitFunc) gdam_sample_editor_skin_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_SKIN;
		sample_editor_skin_type = gtk_type_unique(parent,
		                                 &sample_editor_skin_info);
	}
	return sample_editor_skin_type;
}

static void connect_drawing_area(GtkDrawingArea*       drawing_area,
                                 GdamSampleEditorSkin* skin)
{
	skin->drawing_area = GTK_WIDGET(drawing_area);
	gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), 800, 300);
	gtk_signal_connect(GTK_OBJECT(drawing_area),
	                   "button_press_event",
			   GTK_SIGNAL_FUNC(editor_button_press),
			   skin);
	gtk_signal_connect(GTK_OBJECT(drawing_area),
	                   "button_release_event",
			   GTK_SIGNAL_FUNC(editor_button_release),
			   skin);
	gtk_signal_connect(GTK_OBJECT(drawing_area),
	                   "expose_event",
			   GTK_SIGNAL_FUNC(editor_expose),
			   skin);
	gtk_signal_connect(GTK_OBJECT(drawing_area),
	                   "configure_event",
			   GTK_SIGNAL_FUNC(editor_configure),
			   skin);
	gtk_widget_show(GTK_WIDGET(drawing_area));
	gtk_widget_add_events(GTK_WIDGET(drawing_area),
	                                    GDK_EXPOSURE_MASK
                                          | GDK_ENTER_NOTIFY_MASK
                                          | GDK_LEAVE_NOTIFY_MASK
                                          | GDK_BUTTON_PRESS_MASK
                                          | GDK_BUTTON_RELEASE_MASK
                                          | GDK_POINTER_MOTION_MASK
                                          | GDK_POINTER_MOTION_HINT_MASK);
}
static void redraw_area_now(GdamSampleEditorSkin* skin,
                            GdkRectangle*         rectangle)
{
...
}

static void delayed_redraw(GdamSampleEditorSkin* skin)
{
...
}

/*
 *             _                     _
 *   ___ _   _| |_    __ _ _ __   __| |  ___  __ ___   _____
 *  / __| | | | __|  / _` | '_ \ / _` | / __|/ _` \ \ / / _ \
 * | (__| |_| | |_  | (_| | | | | (_| | \__ \ (_| |\ V /  __/
 *  \___|\__,_|\__|  \__,_|_| |_|\__,_| |___/\__,_| \_/ \___|
 * 
 */

static void cut_and_save_successful(int buffer_id,
                                    GdamSampleEditorSkin* skin)
{
...
}

static void cut_and_save_failed(GdamSampleEditorSkin* skin)
{
...
}

static void cut_and_save_selection(GdamSampleEditorSkin* skin)
{
	/* Issue cut-and-save command to server. */
	...
}

/*
 *             _
 *   ___ _   _| |_
 *  / __| | | | __|
 * | (__| |_| | |_
 *  \___|\__,_|\__|
 * 
 */

static void cut_selection(GdamSampleEditorSkin* skin)
{
...
}

/*
 *                 _
 *  _ __ ___ _ __ | | __ _  ___ ___
 * | '__/ _ \ '_ \| |/ _` |/ __/ _ \
 * | | |  __/ |_) | | (_| | (_|  __/
 * |_|  \___| .__/|_|\__,_|\___\___|
 *          |_|
 */
static void replace_succeeded(GdamSampleEditorSkin* skin)
{...}

static void replace_failed(GdamSampleEditorSkin* skin)
{...}
static void replace_selection(GdamSampleEditorSkin* skin)
{
...
}

/*
 *                                  _   _                __
 *   ___ ___  _ __  _ __   ___  ___| |_(_) ___  _ __    / _|_   _ _ __   ___
 *  / __/ _ \| '_ \| '_ \ / _ \/ __| __| |/ _ \| '_ \  | |_| | | | '_ \ / __|
 * | (_| (_) | | | | | | |  __/ (__| |_| | (_) | | | | |  _| |_| | | | | (__
 *  \___\___/|_| |_|_| |_|\___|\___|\__|_|\___/|_| |_| |_|  \__,_|_| |_|\___|
 * 
 */
void gdam_sample_editor_skin_connect_func(const gchar*     handler,
                                          GtkObject*       object,
		                          const gchar*     signal_name,
			                  const gchar*     signal_data,
		                          GtkObject*       connect_object,
		                          gboolean         after,
		                          gpointer         user_data)
{
	GdamSampleEditorSkin* skin = (GdamSampleEditorSkin*) user_data;
	GdamSampleEditor* editor = GDAM_SAMPLE_EDITOR(get_sample_editor(skin));

	if (strcmp(handler, "gdam_sample_editor_drawing_area") == 0){
		if (!GTK_IS_DRAWING_AREA(object)) {
			g_warning("%s may only be connected to a drawing area",
				handler);
		}
		connect_drawing_area(GTK_DRAWING_AREA(object), skin);
		return;
	}

	if (strcmp(handler, "gdam_sample_editor_cut_and_save") == 0){
		gtk_signal_connect_object(object,
		                          signal_name,
				  GTK_SIGNAL_FUNC(cut_and_save_selection),
					  skin);
		return;
	}

	if (strcmp(handler, "gdam_sample_editor_cut") == 0){
		gtk_signal_connect_object(object,
		                          signal_name,
					  GTK_SIGNAL_FUNC(cut_selection),
					  skin);
		return;
	}

	if (strcmp(handler, "gdam_sample_editor_replace") == 0){
		gtk_signal_connect_object(object,
		                          signal_name,
					  GTK_SIGNAL_FUNC(replace_selection),
					  skin);
		return;
	}

